@@ -1,8 +1,8 @@
 # frozen_string_literal: true
 
-require("mkmf")
+require "mkmf"
 
-if RUBY_ENGINE == "ruby"
+if %w[ruby truffleruby].include?(RUBY_ENGINE)
   $CFLAGS << " -O3 "
   $CFLAGS << " -std=c99"
 
