@@ -1,12 +1,16 @@
 # frozen_string_literal: true
 
-require("test_helper")
+require "test_helper"
 
 module Bootsnap
   module LoadPathCache
-    class ChangeObserverTest < MiniTest::Test
+    class ChangeObserverTest < Minitest::Test
+      include LoadPathCacheHelper
+
       def setup
+        super
         @observer = Object.new
+        @observer.instance_variable_set(:@mutex, Mutex.new)
         @arr = []
         ChangeObserver.register(@arr, @observer)
       end
@@ -25,7 +29,7 @@ module Bootsnap
         @arr.unshift("f", "g")
 
         @observer.expects(:push_paths).with(@arr, "h", "i")
-        @arr.concat(%w(h i))
+        @arr.push("h", "i")
 
         @observer.expects(:unshift_paths).with(@arr, "j", "k")
         @arr.prepend("j", "k")
@@ -42,7 +46,7 @@ module Bootsnap
         @arr.push("b", "c")
         @arr.append("d", "e")
         @arr.unshift("f", "g")
-        @arr.concat(%w(h i))
+        @arr.push("h", "i")
         @arr.prepend("j", "k")
         @arr.delete(3)
         @arr.compact!
@@ -75,6 +79,20 @@ module Bootsnap
         assert_equal(%w(a), @arr)
       end
 
+      def test_dup_returns_ractor_shareable_instance
+        return unless defined?(Ractor)
+
+        ChangeObserver.register(@arr, @observer)
+        Ractor.make_shareable(@arr.dup.freeze)
+      end
+
+      def test_clone_returns_ractor_shareable_instance
+        return unless defined?(Ractor)
+
+        ChangeObserver.register(@arr, @observer)
+        Ractor.make_shareable(@arr.clone.freeze)
+      end
+
       def test_uniq_without_block
         @observer.expects(:reinitialize).never
         @arr.uniq!
