@@ -1,9 +1,11 @@
 # frozen_string_literal: true
 
-require("test_helper")
+require "test_helper"
 
 module Bootsnap
   class KernelRequireTest < Minitest::Test
+    include LoadPathCacheHelper
+
     def test_uses_the_same_duck_type_as_require
       skip("Need a working Process.fork to test in isolation") unless Process.respond_to?(:fork)
       begin
@@ -15,9 +17,9 @@ module Bootsnap
           $LOAD_PATH.push(dir)
           FileUtils.touch("#{dir}/a.rb")
           stringish = mock
-          stringish.expects(:to_str).returns("a").twice # bootsnap + ruby
+          stringish.expects(:to_str).returns("a").at_least(2) # bootsnap + ruby
           pathish = mock
-          pathish.expects(:to_path).returns(stringish).twice # bootsnap + ruby
+          pathish.expects(:to_path).returns(stringish).at_least(2) # bootsnap + ruby
           assert pathish.respond_to?(:to_path)
           require(pathish)
           FileUtils.rm_rf(dir)
@@ -29,9 +31,28 @@ module Bootsnap
         cache.unlink
       end
     end
+
+    def test_load_static_libaries
+      skip("Need a working Process.fork to test in isolation") unless Process.respond_to?(:fork)
+      skip("Need some libraries to be compiled statically") unless RUBY_VERSION >= "3.3"
+
+      begin
+        assert_nil LoadPathCache.load_path_cache
+        cache = Tempfile.new("cache")
+        pid = Process.fork do
+          LoadPathCache.setup(cache_path: cache, development_mode: false, ignore_directories: nil)
+          require("prism")
+        end
+        _, status = Process.wait2(pid)
+        assert_predicate status, :success?
+      ensure
+        cache.close
+        cache.unlink
+      end
+    end
   end
 
-  class KernelLoadTest < MiniTest::Test
+  class KernelLoadTest < Minitest::Test
     def setup
       @initial_dir = Dir.pwd
       @dir1 = File.realpath(Dir.mktmpdir)
